@@ -53,6 +53,7 @@ import org.jruby.runtime.CallType;
 import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.ContextAwareBlockBody;
 import org.jruby.runtime.Helpers;
+import org.jruby.runtime.JavaSites;
 import org.jruby.runtime.MethodIndex;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.Signature;
@@ -71,6 +72,7 @@ import org.jruby.util.SymbolNameType;
 import org.jruby.util.TypeConverter;
 
 import java.lang.ref.WeakReference;
+import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;
 import java.util.concurrent.locks.ReentrantLock;
 
 import static org.jruby.util.RubyStringBuilder.str;
@@ -93,7 +95,8 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
     private final ByteList symbolBytes;
     private final int hashCode;
     private String decodedString;
-    private RubyString rubyString;
+    private volatile RubyString rubyString;
+    private static final AtomicReferenceFieldUpdater<RubySymbol, RubyString> RUBY_STRING_UPDATER = AtomicReferenceFieldUpdater.newUpdater(RubySymbol.class, RubyString.class, "rubyString");
     private transient Object constant;
     private SymbolNameType type;
 
@@ -472,7 +475,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
 
     @JRubyMethod(name = "to_s")
     public IRubyObject to_s(ThreadContext context) {
-        return to_s(context.runtime);
+        return getNameString(context).strDup(context.runtime);
     }
 
     final RubyString to_s(Ruby runtime) {
@@ -481,22 +484,38 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
 
     @JRubyMethod(name = "name")
     public IRubyObject name(ThreadContext context) {
-      // FIXME: A race here where two in-flight to_s on same symbol can return a different instance.
-      if (rubyString == null) {
-        rubyString = RubyString.newStringShared(context.runtime, getBytes());
-        rubyString.setFrozen(true);
-      }
-
-      return rubyString;
+        return getNameString(context);
     }
 
+    private RubyString getNameString(ThreadContext context) {
+        RubyString currentString = rubyString;
+
+        if (currentString != null) {
+            return currentString;
+        }
+
+        return cacheNameString(context);
+    }
+
+    private RubyString cacheNameString(ThreadContext context) {
+        // atomically create and store an fstring for this symbol
+        Ruby runtime = context.runtime;
+        RubyString nameString = RubyString.newStringShared(runtime, getBytes());
+        nameString.scanForCodeRange();
+        nameString = runtime.freezeAndDedupString(nameString);
+        RUBY_STRING_UPDATER.compareAndSet(this, null, nameString);
+
+        return rubyString;
+    }
+
+    @Deprecated
     public IRubyObject id2name() {
-        return to_s(metaClass.runtime);
+        return name(getRuntime().getCurrentContext());
     }
 
     @JRubyMethod
     public IRubyObject id2name(ThreadContext context) {
-        return to_s(context);
+        return name(context);
     }
 
     @Override
@@ -995,8 +1014,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         public SymbolTable(Ruby runtime) {
             this.runtime = runtime;
             this.loadFactor = DEFAULT_LOAD_FACTOR;
-            this.threshold = (int)(DEFAULT_INITIAL_CAPACITY * DEFAULT_LOAD_FACTOR);
-            this.symbolTable = new SymbolEntry[DEFAULT_INITIAL_CAPACITY];
+            reset();
         }
 
         // note all fields are final -- rehash creates new entries when necessary.
@@ -1336,6 +1354,15 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
             return size;
         }
 
+        public void clear() {
+            reset();
+        }
+
+        private void reset() {
+            this.threshold = (int)(DEFAULT_INITIAL_CAPACITY * DEFAULT_LOAD_FACTOR);
+            this.symbolTable = new SymbolEntry[DEFAULT_INITIAL_CAPACITY];
+        }
+
         private SymbolEntry[] rehash() {
             SymbolEntry[] oldTable = symbolTable;
             int oldCapacity = oldTable.length;
@@ -1471,21 +1498,48 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         return object.convertToString().getByteList().toString();
     }
 
-    // MRI: rb_check_id
+    @Deprecated
     public static String checkID(IRubyObject object) {
-        if (object instanceof RubySymbol) return ((RubySymbol) object).idString();
+        return idStringFromObject(object.getRuntime().getCurrentContext(), object);
+    }
 
-        if (!(object instanceof RubyString)) {
-            IRubyObject tmp = TypeConverter.checkStringType(object.getRuntime(), object);
+    // MRI: rb_check_id but producing a Java String
+    public static String idStringFromObject(ThreadContext context, IRubyObject object) {
+        IRubyObject symOrStr = prepareID(object.getRuntime().getCurrentContext(), object);
 
-            if (tmp.isNil()) {
-                throw object.getRuntime().newTypeError(str(object.getRuntime(), "", object, " is not a symbol nor a string"));
-            }
+        if (symOrStr instanceof RubySymbol) return ((RubySymbol) symOrStr).idString();
 
-            object = tmp;
+        return ((RubyString) symOrStr).getByteList().toString();
+    }
+
+    // MRI: rb_check_id
+    public static RubySymbol idSymbolFromObject(ThreadContext context, IRubyObject object) {
+        IRubyObject symOrStr = prepareID(context, object);
+
+        if (symOrStr instanceof RubySymbol) return (RubySymbol) symOrStr;
+
+        return newSymbol(context.runtime, ((RubyString) symOrStr).getByteList());
+    }
+
+    /**
+     * Return the given object if it is a Symbol or String, or convert it to a String.
+     *
+     * @param context the current context
+     * @param object the object
+     * @return the object, if it is a Symbol or String, or a String produced by calling #to_str on the object.
+     */
+    public static IRubyObject prepareID(ThreadContext context, IRubyObject object) {
+        if (object instanceof RubySymbol || object instanceof RubyString) return object;
+
+        Ruby runtime = context.runtime;
+
+        IRubyObject tmp = TypeConverter.checkStringType(context, sites(context).to_str_checked, object);
+
+        if (tmp.isNil()) {
+            throw runtime.newTypeError(str(runtime, "", object, " is not a symbol nor a string"));
         }
 
-        return ((RubyString) object).getByteList().toString();
+        return tmp;
     }
 
     public static final class SymbolProcBody extends ContextAwareBlockBody {
@@ -1571,6 +1625,10 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         }
     }
 
+    private static JavaSites.SymbolSites sites(ThreadContext context) {
+        return context.sites.Symbol;
+    }
+
     @Deprecated
     @Override
     public IRubyObject taint(ThreadContext context) {
