@@ -188,9 +188,6 @@ public class RubySet extends RubyObject implements Set {
      */
     @JRubyMethod(visibility = Visibility.PRIVATE) // def initialize(enum = nil, &block)
     public IRubyObject initialize(ThreadContext context, Block block) {
-        if ( block.isGiven() && context.runtime.isVerbose() ) {
-            context.runtime.getWarnings().warning(IRubyWarnings.ID.BLOCK_UNUSED, "given block not used");
-        }
         allocHash(context.runtime);
         return this;
     }
@@ -924,6 +921,13 @@ public class RubySet extends RubyObject implements Set {
     @Override
     @JRubyMethod
     public RubyFixnum hash() { // @hash.hash
+        RubyHash hash = this.hash;
+
+        if (hash == null) {
+            // Emulate set.rb for jruby/jruby#8393
+            return ((RubyBasicObject) getRuntime().getNil()).hash();
+        }
+
         return hash.hash();
     }
 
