@@ -16,9 +16,10 @@ import org.jruby.RubyFloat;
 import org.jruby.RubyInteger;
 import org.jruby.RubyProc;
 import org.jruby.RubyString;
+import org.jruby.RubySymbol;
 import org.jruby.java.invokers.RubyToJavaInvoker;
+import org.jruby.java.util.ClassUtils;
 import org.jruby.javasupport.JavaCallable;
-import org.jruby.javasupport.JavaClass;
 import org.jruby.javasupport.JavaUtil;
 import org.jruby.javasupport.ParameterTypes;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -606,7 +607,7 @@ public class CallableSelector {
     }
 
     private static boolean assignable(Class<?> type, final IRubyObject arg) {
-        return JavaClass.assignable(type, getJavaClass(arg)) ||
+        return ClassUtils.assignable(type, getJavaClass(arg)) ||
                 // handle 'native' signatures e.g. method with a (org.jruby.RubyArray arg)
                 ( arg != null && type.isAssignableFrom(arg.getClass()) );
     }
@@ -706,6 +707,10 @@ public class CallableSelector {
             if ( CharSequence.class.isAssignableFrom(type) ) return 7;
             if ( type == Character.class ) return 1;
         }
+        else if ( arg instanceof RubySymbol ) {
+            if ( type == String.class ) return 10;
+            if ( CharSequence.class.isAssignableFrom(type) ) return 7;
+        }
         else if ( arg instanceof RubyBoolean ) {
             if ( type == Boolean.class ) return 10;
             //if ( type == Byte.class ) return 2;
@@ -859,4 +864,4 @@ public class CallableSelector {
         return method;
     }
 
-}
\ No newline at end of file
+}
