@@ -62,10 +62,8 @@ import org.jruby.RubyThread;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.exceptions.CatchThrow;
 import org.jruby.exceptions.JumpException;
-import org.jruby.exceptions.LoadError;
 import org.jruby.exceptions.MainExitException;
 import org.jruby.exceptions.RaiseException;
-import org.jruby.exceptions.StandardError;
 import org.jruby.exceptions.Unrescuable;
 import org.jruby.ext.rbconfig.RbConfigLibrary;
 import org.jruby.platform.Platform;
@@ -1009,4 +1007,9 @@ public class LoadService {
 
         return filename;
     }
+
+    public void tearDown() {
+        loadedFeatures.clear();
+        librarySearcher.tearDown();
+    }
 }
