@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 # two jars with embedded gems
 jar 'org.rubygems:gem1', '1'
 jar 'org.rubygems:gem2', '2'
@@ -6,6 +8,6 @@ jar 'org.rubygems:gem2', '2'
 pom 'org.jruby:jruby', '${jruby.version}'
 
 # unit tests
-jar 'junit:junit', '4.8.2', :scope => :test
+jar 'junit:junit', '4.8.2', scope: :test
 
-resource :directory => "src/main/ruby"
+resource directory: 'src/main/ruby'
