@@ -10,9 +10,15 @@ exclude :test_strftime_invalid_flags, "needs investigation"
 exclude :test_strftime_no_hidden_garbage, "requires ObjectSpace count_objects"
 exclude :test_strftime_rational, "needs investigation"
 exclude :test_strftime_seconds_from_epoch, "Integer unification, needs Bignum support (#4687)"
-exclude :test_strftime_too_wide, "needs investigation"
 exclude :test_strftime_year, "Integer unification, needs Bignum support (#4687)"
 exclude :test_utc_or_local, "needs investigation"
 exclude :test_zone, "not implemented, zone = 0 should not return UTC"
 exclude :test_2038, "OK: RangeError: bignum too big to convert into `long'"
+exclude :test_getlocal_utc, "needs investigation"
+exclude :test_inspect, "needs investigation"
+exclude :test_strfimte_zoneoffset, "needs investigation"
 exclude :test_strftime_wide_precision, "needs investigation"
+exclude :test_new, "needs investigation"
+exclude :test_strftime, "needs investigation"
+exclude :test_marshal_distant_future, "needs investigation"
+exclude :test_marshal_distant_past, "needs investigation"
